/*
    How to add a new entry for a method in the apiXX.js files' JSON objects:
    
    - each resourse is described in the form of
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    
        "[RESOURCE_1]" : {
            "[SUB-RESOURCE_A]" : {
                "path" : "[PATH_TO_RERSOURCE] (HUMAN_REABABLE: includes the slashes and variable placeholders, i.e. /applications/{name})]",
                "methods" : {
                    "[METHOD_A]" : {
                        "request" : ["param1", "param2", ....],
                        "response" : "[RESPONSE_JSON] (One line, escaped quotes, and the tabs and new lines are replaced with text sequences i.e. \t, \n and \")"
                    },
                    "[METHOD_B]" : {
                        "request" : ["param1", "param2", ....],
                        "response" : "[RESPONSE_JSON]"
                    },
                    ..... further methods
                }
            },
            "[SUB-RESOURCE_B]" : {
                "path" : "[PATH_TO_RERSOURCE]",
                "methods" : {
                    "[METHOD_A]" : {
                        "request" : ["param1", "param2", ....],
                        "response" : "[RESPONSE_JSON]"
                    },
                    "[METHOD_B]" : {
                        "request" : ["param1", "param2", ....],
                        "response" : "[RESPONSE_JSON]"
                    },
                    .....
                }
            },
            ..... further sub-resources
    },
    "[RESOURCE_2]" : {
        .....
    },
    .... further resources
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    
    - you need the list of allowed request params and put them into array in each method's "request" element.
    See https://confluence.axway.com/display/ST/%5BDRAFT%5D+Deprecated+API+versions+migration+document
    
    - you will need a complete JSON object of the response. See how it is shown in 
    https://confluence.axway.com/display/ST/%5BDRAFT%5D+Deprecated+API+versions+migration+document
    
    - to compress the response JSON for the "response" elements, you need to:
    
        - no empty lines
        - use proper tabs - in the examples from Jira/Confluence, two spaces were one tab, so replace
        them with \t
        - replace the new lines with \n
        - then compress the final text bloc into one line (multiple text editors allow it as function)
        
    - then add the new element's structure in the apiXX.js file
    
    - each apiXX.js file must be inclided in the HEAD element of the HTML file
    
    - the main JS function generate_content() uses the old API's version tag (i.e. 10 for 1.0, 11 for 1.1 etc)
    as call parameter to read the respective apiXX.js object and compare it to the api20.js object. Thus,
    you will end up having 4 calls to the function: 
    
        generate_content(10);
        generate_content(11);
        generate_content(12);
        generate_content(14);
  
    - in the displayed request params fields, the params, which match in API 2.0 the ones in API 1.x, will be
    coloured. CSS style class is .matching
       
    
*/



var htmltab = "&#160;&#160;&#160;&#160;";

jQuery(document).ready( function () {

    // main tabs navigation and tabs control
    jQuery("#navi-wrapper").on("click", "#navi-links li", function( event ) {
        navi_actions( jQuery(this) );
    });
    
    jQuery("#content-tabs").on("click", ".swag-title", function( event ) {
        toggle_resource_actions( jQuery(this) );
    });
    //document.querySelector("#js-resource-10-accounts > div:nth-child(1) > div.resource-title.get")
    jQuery("#content-tabs ").on("click", "div.resource-title", function( event ) {
        toggle_method_actions( jQuery(this) );
    });
	
    generate_content(10, "1.0");
	generate_content(11, "1.1");
    generate_content(12, "1.2");
    generate_content(13, "1.3");
    
});

function toggle_resource_actions( pJqObj ) {

    var skey = pJqObj.attr('data-link-group'); 
    var rel = pJqObj.attr('rel');
    var parentTab = "#tab-" + skey;
    var targetBody = "#" + rel;
    jQuery( targetBody ).toggle();

}

function toggle_method_actions( pJqObj ) {
    var rel = pJqObj.attr('rel');
    var targetBody = "#" + rel;
    
    pJqObj.toggleClass("method-open");
    jQuery( targetBody ).toggleClass("invisible");
}

function navi_actions( pJqObj ) {

    var skey = pJqObj.attr('data-link-group');    
    var targetTab = "#tab-" + skey;
    var targetNav = "#nav-" + skey;
    jQuery( "#content-tabs div.js-tabs" ).addClass( "invisible" );
    jQuery( targetTab ).removeClass("invisible");
    
    jQuery( "#navi-links li.js-nav" ).removeClass( "active" );
    jQuery( targetNav ).addClass( "active" );
        
}

function generate_content( pApiVer, sApiVer ) {

    var workingApi = api[pApiVer];

	console.log(workingApi);
    var targetTab = "#tab-" + pApiVer;
    var resources =  Object.keys(workingApi).sort();
    
    console.log(resources);
    var H = "";
    
    jQuery.each(resources, function ( index, resource ) {
        
        
        var resData = workingApi[resource];
        var resIdTag = "js-resource-"+ pApiVer +"-"+ resource;
        var endpoints = Object.keys(resData).sort();
        console.log(endpoints);
        var E = "";
        jQuery.each(endpoints, function ( eindex, endpoint ) {
        
            var methods = Object.keys(resData[endpoint]["methods"]);
            var M = "";
            
            jQuery.each(methods, function ( mindex, method ) {
            
                var mTag = "method-"+ pApiVer + "-" + endpoint + "-" + mindex;
				
                var method1x = workingApi[resource][endpoint]["methods"][method];
				var method1420 = null;
				var targetAPI = "2.0";
				
				
				if(api20[resource] && api20[resource][endpoint] && !(typeof api20[resource][endpoint]["methods"][method] === "undefined")){
					method1420 = api20[resource][endpoint]["methods"][method];
				}else if(api14[resource] && api14[resource][endpoint] && !(typeof api14[resource][endpoint]["methods"][method] === "undefined")){
					method1420 = api14[resource][endpoint]["methods"][method];
					targetAPI = "1.4";
				}else{
					targetAPI = "Not Available";
				}
			
                
                var req1x = "";
                var resp1x = char_encode(method1x["response"]);
                jQuery.each(method1x["request"], function ( r1, param1 ) {
                    req1x += (method1420 == null || !__in_arr( param1, method1420["request"]) ? "<span class='not-matching'>"+ param1 +"</span>" : param1) + "<br>";
                });
				var req1xBody = "request_body" in method1x ? "<div class='bold-title quarter-title'>Body</div>" + "<div class='quarter-body code'>" + char_encode(method1x["request_body"]) + "</div>" : "";
				
				var req20 = "";
				var resp20 = "";
				var endpoint20 = "";
				var req20Body = "";
				if (method1420 != null) {
					resp20 = char_encode(method1420["response"]);
					endpoint20 = " <span class='font-title'>" + ("method" in method1420 ? method1420["method"] : "").toUpperCase().trimEnd() + " " + ("path" in method1420 ? method1420["path"] : "") + "</span>";
					jQuery.each(method1420["request"], function ( r2, param2 ) {
						req20 += (__in_arr( param2, method1x["request"]) ? "<span class='matching'>"+ param2 +"</span>" : param2) + "<br>";
					});
					req20Body = "request_body" in method1420 ? "<div class='quarter-title bold-title'>Body</div>" + "<div class='quarter-body code'>" + char_encode(method1420["request_body"]) + "</div>" : "";
				}
                
                M += "<div class='resource-title " + method + "' rel='" + mTag + "'>"
                    +      "<div class='methods method-" + method + "'>" + method + "</div>"
                    +      "<div class='resource-url'>" + resData[endpoint]["path"] + "</div>"
					+      "<div class='resource-description'>" + resData[endpoint]["methods"][method]["description"] + "</div>"
                    + "</div>"
                
                    + "<div class='resource-body " + method + " invisible' id='" + mTag + "'>"
                    +     "<div class='resource-half'>"
                    +         "<div class='half-title'>Request</div>"
                    +         "<div class='half-body'>"
                    +             "<div class='quarter'>"
                    +                 "<div class='quarter-title'>API " + sApiVer + "</div>"
					+ 				  "<div class='bold-title quarter-title'>Parameters</div>"
                    +                 "<div class='quarter-body code'>"
                    +                     req1x
                    +                "</div>"
                    +                req1xBody
                    +             "</div>"
                    +             "<div class='quarter'>"
                    +                 "<div class='quarter-title'>API " + targetAPI 
					+					  endpoint20
					+				  "</div>"
					+ 				  "<div class='bold-title quarter-title'>Parameters</div>"
                    +                 "<div class='quarter-body code'>"
                    +                     req20
                    +                 "</div>"
					+ 				  req20Body
                    +             "</div>"
                    +         "</div>"
                    +     "</div>"
                    +     "<div class='resource-half'>"
                    +         "<div class='half-title'>Response</div>"
                    +         "<div class='half-body'>"
                    +             "<div class='quarter'>"
                    +                 "<div class='quarter-title'>API " + sApiVer + "</div>"
					+				  "<div class='bold-title quarter-title'>&nbsp;</div>"
                    +                "<div class='quarter-body code'>" + resp1x + "</div>"
                    +             "</div>                                            "
                    +             "<div class='quarter'>"
                    +                 "<div class='quarter-title'>API " + targetAPI + "</div>"
					+				  "<div class='bold-title quarter-title'>&nbsp;</div>"
                    +                 "<div class='quarter-body code'>" + resp20 + "</div>"
                    +             "</div>"
                    +         "</div>"
                    +     "</div>"
                    + "</div>";
            });
            
            E += "<div class='resource-method'>" + M + "</div>";
            
            
        });
        
        
        H += "<div class='resource-wrapper'>"
            +   "<div class='swag-title' data-link-group='"+ pApiVer +"' rel='"+ resIdTag +"'>"+ resource +" Resource</div>"
            +   "<div class='swag-body' id='" + resIdTag + "'>"
            +   E
            +   "</div>"   
            +"</div>"
            ;
        
    });
        
    jQuery(targetTab).html(H);
    
    
}

function __in_arr( pValue, pArray ) {
    return ( jQuery.inArray( pValue, pArray ) > -1 );
}

function char_encode( pText ) {
	type = Object.prototype.toString.call(pText);
	console.log(type);
	if (type != "[object String]") {
		result = JSON.stringify(pText, null, "\t");
	} else {
		result = pText;		
	}
    return result.replace(/\t/g, htmltab).replace(/\n/g, "<br>");
}
